@@ -4,8 +4,9 @@ use strict;
 use base 'Class::Data::Inheritable';
 use File::MMagic;
 use File::Slurp;
+use File::stat;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 __PACKAGE__->mk_classdata('mmagic');
 __PACKAGE__->mmagic( File::MMagic->new );
@@ -37,10 +38,31 @@ Serve static files from config->{root}.
 sub serve_static {
     my $c    = shift;
     my $path = $c->config->{root} . '/' . $c->req->path;
+    return $c->serve_static_file( $path, @_ );
+}
+
+sub serve_static_file {
+    my $c    = shift;
+    my $path = shift;
+    
     if ( -f $path ) {
+
+        my $stat = stat($path);
+
+        if ( $c->req->headers->header('If-Modified-Since') ) {
+
+            if ( $c->req->headers->if_modified_since == $stat->mtime ) {
+                $c->res->status(304); # Not Modified
+                $c->res->headers->remove_content_headers;
+                return 1;
+            }
+        }
+
         my $content = read_file($path);
         my $type = shift || __PACKAGE__->mmagic->checktype_contents($content);
         $c->res->headers->content_type($type);
+        $c->res->headers->content_length( $stat->size );
+        $c->res->headers->last_modified( $stat->mtime );
         $c->res->output($content);
         $c->log->debug(qq/Serving file "$path" as "$type"/) if $c->debug;
         return 1;
@@ -55,6 +77,7 @@ L<Catalyst>.
 =head1 AUTHOR
 
 Sebastian Riedel, C<sri@cpan.org>
+Christian Hansen <ch@ngmedia.com>
 
 =head1 THANK YOU
 
